@@ -1,31 +1,41 @@
-"""
-GillesPy2 is a modeling toolkit for biochemical simulation.
-Copyright (C) 2019-2021 GillesPy2 developers.
+# GillesPy2 is a modeling toolkit for biochemical simulation.
+# Copyright (C) 2019-2022 GillesPy2 developers.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
-
-from gillespy2.core.jsonify import TranslationTable
-from gillespy2.core.reaction import *
-from gillespy2.core.raterule import RateRule
-from gillespy2.core.parameter import Parameter
-from gillespy2.core.species import Species
-from gillespy2.core.reaction import Reaction
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import numpy as np
-from gillespy2.core.results import Trajectory,Results
+from typing import Set, Type
 from collections import OrderedDict
-from gillespy2.core.gillespyError import *
+
+import gillespy2
+from gillespy2.core.assignmentrule import AssignmentRule
+from gillespy2.core.events import Event
+from gillespy2.core.functiondefinition import FunctionDefinition
+from gillespy2.core.parameter import Parameter
+from gillespy2.core.raterule import RateRule
+from gillespy2.core.reaction import Reaction
+from gillespy2.core.species import Species
+from gillespy2.core.timespan import TimeSpan
+from gillespy2.core.sortableobject import SortableObject
+from gillespy2.core.jsonify import Jsonify, TranslationTable
+from gillespy2.core.results import Trajectory, Results
+from gillespy2.core.gillespyError import (
+    ParameterError,
+    ModelError,
+    SimulationError,
+    StochMLImportError,
+    InvalidStochMLError
+)
 
 try:
     import lxml.etree as eTree
@@ -168,9 +178,10 @@ class Model(SortableObject, Jsonify):
         else:
             self.units = "concentration"
             if volume != 1.0:
-                raise Warning(
+                raise ModelError(
                     "Concentration models account for volume implicitly, explicit volume definition is not required. "
-                    "Note: concentration models may only be simulated deterministically.")
+                    "Note: concentration models may only be simulated deterministically."
+                )
 
         self.volume = volume
 
@@ -179,8 +190,7 @@ class Model(SortableObject, Jsonify):
         self.namespace = OrderedDict([])
 
         if tspan is None:
-            self.timespan(np.linspace(0, 20, 401))
-            self.user_set_tspan = False
+            self.tspan = None
         else:
             self.timespan(tspan)
 
@@ -227,6 +237,55 @@ class Model(SortableObject, Jsonify):
 
         return print_string
 
+    def add(self, components):
+        """
+        Adds a component, or list of components to the model. If a list is provided, Species
+        and Parameters are added before other components.  Lists may contain any combination
+        of accepted types other than lists and do not need to be in any particular order.
+
+        :param components: The component or list of components to be added the the model.
+        :type components: Species, Parameters, Reactions, Events, Rate Rules, Assignment Rules, \
+                          FunctionDefinitions, and TimeSpan or list
+
+        :returns: The components that were added to the model.
+        :rtype: Species, Parameters, Reactions, Events, Rate Rules, Assignment Rules, \
+                FunctionDefinitions, and TimeSpan or list
+
+        :raises ModelError: Component is invalid.
+        """
+        if isinstance(components, list):
+            p_types = (Species, Parameter, FunctionDefinition, TimeSpan)
+            p_names = (p_type.__name__ for p_type in p_types)
+
+            others = []
+            for component in components:
+                if isinstance(component, p_types) or type(component).__name__ in p_names:
+                    self.add(component)
+                else:
+                    others.append(component)
+
+            for component in others:
+                self.add(component)
+        elif isinstance(components, AssignmentRule) or type(components).__name__ == AssignmentRule.__name__:
+            self.add_assignment_rule(components)
+        elif isinstance(components, Event) or type(components).__name__ == Event.__name__:
+            self.add_event(components)
+        elif isinstance(components, FunctionDefinition) or type(components).__name__ == FunctionDefinition.__name__:
+            self.add_function_definition(components)
+        elif isinstance(components, Parameter) or type(components).__name__ == Parameter.__name__:
+            self.add_parameter(components)
+        elif isinstance(components, RateRule) or type(components).__name__ == RateRule.__name__:
+            self.add_rate_rule(components)
+        elif isinstance(components, Reaction) or type(components).__name__ == Reaction.__name__:
+            self.add_reaction(components)
+        elif isinstance(components, Species) or type(components).__name__ == Species.__name__:
+            self.add_species(components)
+        elif isinstance(components, TimeSpan) or type(components).__name__ == TimeSpan.__name__:
+            self.timespan(components)
+        else:
+            raise ModelError(f"Unsupported component: {type(components)} is not a valid component.")
+        return components
+
     def make_translation_table(self):
         from collections import ChainMap
 
@@ -280,28 +339,28 @@ class Model(SortableObject, Jsonify):
 
     def problem_with_name(self, name):
         if name in Model.reserved_names:
-            return ModelError(
+            raise ModelError(
                 'Name "{}" is unavailable. It is reserved for internal GillesPy use. Reserved Names: ({}).'.format(name,
                                                                                                                    Model.reserved_names))
         if name in self.listOfSpecies:
-            return ModelError('Name "{}" is unavailable. A species with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. A species with that name exists.'.format(name))
         if name in self.listOfParameters:
-            return ModelError('Name "{}" is unavailable. A parameter with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. A parameter with that name exists.'.format(name))
         if name in self.listOfReactions:
-            return ModelError('Name "{}" is unavailable. A reaction with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. A reaction with that name exists.'.format(name))
         if name in self.listOfEvents:
-            return ModelError('Name "{}" is unavailable. An event with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. An event with that name exists.'.format(name))
         if name in self.listOfRateRules:
-            return ModelError('Name "{}" is unavailable. A rate rule with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. A rate rule with that name exists.'.format(name))
         if name in self.listOfAssignmentRules:
-            return ModelError('Name "{}" is unavailable. An assignment rule with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. An assignment rule with that name exists.'.format(name))
         if name in self.listOfFunctionDefinitions:
-            return ModelError('Name "{}" is unavailable. A function definition with that name exists.'.format(name))
+            raise ModelError('Name "{}" is unavailable. A function definition with that name exists.'.format(name))
         if name.isdigit():
-            return ModelError('Name "{}" is unavailable. Names must not be numeric strings.'.format(name))
+            raise ModelError('Name "{}" is unavailable. Names must not be numeric strings.'.format(name))
         for special_character in Model.special_characters:
             if special_character in name:
-                return ModelError(
+                raise ModelError(
                     'Name "{}" is unavailable. Names must not contain special characters: {}.'.format(name,
                                                                                                       Model.special_characters))
 
@@ -334,24 +393,23 @@ class Model(SortableObject, Jsonify):
                 self.add_species(S)
         else:
             try:
-                problem = self.problem_with_name(obj.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(obj.name)
                 self.listOfSpecies[obj.name] = obj
                 self._listOfSpecies[obj.name] = 'S{}'.format(len(self._listOfSpecies))
             except Exception as e:
                 raise ParameterError("Error using {} as a Species. Reason given: {}".format(obj, e))
         return obj
 
-    def delete_species(self, obj):
+    def delete_species(self, name):
         """
         Removes a species object by name.
 
-        :param obj: Name of the species object to be removed
-        :type obj: str
+        :param name: Name of the species object to be removed
+        :type name: str
         """
-        self.listOfSpecies.pop(obj)
-        self._listOfSpecies.pop(obj)
+        self.listOfSpecies.pop(name)
+        if name in self._listOfSpecies:
+            self._listOfSpecies.pop(name)
 
     def delete_all_species(self):
         """
@@ -417,9 +475,7 @@ class Model(SortableObject, Jsonify):
                 self.add_parameter(p)
         else:
             if isinstance(params, Parameter) or type(params).__name__ == 'Parameter':
-                problem = self.problem_with_name(params.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(params.name)
                 self.update_namespace()
                 params._evaluate(self.namespace)
                 self.listOfParameters[params.name] = params
@@ -428,15 +484,16 @@ class Model(SortableObject, Jsonify):
                 raise ParameterError("Parameter {}  must be of type {}, it is of type {}".format(params, str(type(Parameter)), str(params) ))
         return params
 
-    def delete_parameter(self, obj):
+    def delete_parameter(self, name):
         """
         Removes a parameter object by name.
 
-        :param obj: Name of the parameter object to be removed
-        :type obj: str
+        :param name: Name of the parameter object to be removed
+        :type name: str
         """
-        self.listOfParameters.pop(obj)
-        self._listOfParameters.pop(obj)
+        self.listOfParameters.pop(name)
+        if name in self._listOfParameters:
+            self._listOfParameters.pop(name)
 
     def set_parameter(self, p_name, expression):
         """
@@ -500,9 +557,7 @@ class Model(SortableObject, Jsonify):
                 self.add_reaction(r)
         else:
             try:
-                problem = self.problem_with_name(reactions.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(reactions.name)
                 reactions.verify()
                 self.validate_reactants_and_products(reactions)
                 if reactions.name is None or reactions.name == '':
@@ -542,9 +597,7 @@ class Model(SortableObject, Jsonify):
                 self.add_rate_rule(rr)
         else:
             try:
-                problem = self.problem_with_name(rate_rules.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(rate_rules.name)
                 if len(self.listOfAssignmentRules) != 0:
                     for i in self.listOfAssignmentRules.values():
                         if rate_rules.variable == i.variable:
@@ -587,9 +640,7 @@ class Model(SortableObject, Jsonify):
                 self.add_event(e)
         else:
             try:
-                problem = self.problem_with_name(event.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(event.name)
                 if event.trigger is None or not hasattr(event.trigger, 'expression'):
                     raise ModelError(
                         'An Event must contain a valid trigger.')
@@ -614,9 +665,7 @@ class Model(SortableObject, Jsonify):
                 self.add_function_definition(fd)
         else:
             try:
-                problem = self.problem_with_name(function_definitions.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(function_definitions.name)
                 self.listOfFunctionDefinitions[function_definitions.name] = function_definitions
             except Exception as e:
                 raise ParameterError(
@@ -634,9 +683,7 @@ class Model(SortableObject, Jsonify):
                 self.add_assignment_rule(ar)
         else:
             try:
-                problem = self.problem_with_name(assignment_rules.name)
-                if problem is not None:
-                    raise problem
+                self.problem_with_name(assignment_rules.name)
                 if len(self.listOfRateRules) != 0:
                     for i in self.listOfRateRules.values():
                         if assignment_rules.variable == i.variable:
@@ -663,19 +710,13 @@ class Model(SortableObject, Jsonify):
         timespans. 
 
         :param time_span: Evenly-spaced list of times at which to sample the species populations during the simulation. 
-            Best to use the form np.linspace(<start time>, <end time>, <number of time-points, inclusive>)
-        :type time_span: numpy ndarray
-        """
-        
-        first_diff = time_span[1] - time_span[0]
-        other_diff = time_span[2:] - time_span[1:-1]
-        isuniform = np.isclose(other_diff, first_diff).all()
-
-        if isuniform:
-            self.user_set_tspan = True
+            Best to use the form gillespy2.TimeSpan(np.linspace(<start time>, <end time>, <number of time-points, inclusive>))
+        :type time_span: gillespy2.TimeSpan | iterator
+        """        
+        if isinstance(time_span, TimeSpan) or type(time_span).__name__ == "TimeSpan":
             self.tspan = time_span
         else:
-            raise InvalidModelError("StochKit only supports uniform timespans")
+            self.tspan = TimeSpan(time_span)
 
     def get_reaction(self, rname):
         """
@@ -690,12 +731,16 @@ class Model(SortableObject, Jsonify):
         """
         return self.listOfReactions
 
-    def delete_reaction(self, obj):
+    def delete_reaction(self, name):
         """
-        :param obj: Name of Reaction to be removed
+        Removes a reaction object by name.
+
+        :param name: Name of Reaction to be removed,
+        :type name: str
         """
-        self.listOfReactions.pop(obj)
-        self._listOfReactions.pop(obj)
+        self.listOfReactions.pop(name)
+        if name in self._listOfReactions:
+            self._listOfReactions.pop(name)
 
     def delete_all_reactions(self):
         """
@@ -717,14 +762,16 @@ class Model(SortableObject, Jsonify):
         """
         return self.listOfEvents
 
-    def delete_event(self, ename):
+    def delete_event(self, name):
         """
         Removes specified Event from model
 
-        :param ename: Name of Event to be removed
+        :param name: Name of Event to be removed.
+        :type name: str
         """
-        self.listOfEvents.pop(ename)
-        self._listOfEvents.pop(ename)
+        self.listOfEvents.pop(name)
+        if name in self._listOfEvents:
+            self._listOfEvents.pop(name)
 
     def delete_all_events(self):
         """
@@ -746,13 +793,16 @@ class Model(SortableObject, Jsonify):
         """
         return self.listOfRateRules
 
-    def delete_rate_rule(self, rname):
+    def delete_rate_rule(self, name):
         """
         Removes specified Rate Rule from model
-        :param rname: Name of Rate Rule to be removed
+
+        :param name: Name of Rate Rule to be removed.
+        :type name: str
         """
-        self.listOfRateRules.pop(rname)
-        self._listOfRateRules.pop(rname)
+        self.listOfRateRules.pop(name)
+        if name in self._listOfRateRules:
+            self._listOfRateRules.pop(name)
 
     def delete_all_rate_rules(self):
         """
@@ -774,14 +824,16 @@ class Model(SortableObject, Jsonify):
         """
         return self.listOfAssignmentRules
 
-    def delete_assignment_rule(self, aname):
+    def delete_assignment_rule(self, name):
         """
         Removes an assignment rule from a model
 
-        :param aname: Name of AssignmentRule object to be removed from model
+        :param name: Name of AssignmentRule object to be removed from model.
+        :type name: str
         """
-        self.listOfAssignmentRules.pop(aname)
-        self._listOfAssignmentRules.pop(aname)
+        self.listOfAssignmentRules.pop(name)
+        if name in self._listOfAssignmentRules:
+            self._listOfAssignmentRules.pop(name)
 
     def delete_all_assignment_rules(self):
         """
@@ -803,14 +855,16 @@ class Model(SortableObject, Jsonify):
         """
         return self.listOfFunctionDefinitions
 
-    def delete_function_definition(self, fname):
+    def delete_function_definition(self, name):
         """
         Removes specified Function Definition from model
 
-        :param fname: Name of Function Definition to be removed
+        :param name: Name of Function Definition to be removed
+        :type name: str
         """
-        self.listOfFunctionDefinitions.pop(fname)
-        self._listOfFunctionDefinitions.pop(fname)
+        self.listOfFunctionDefinitions.pop(name)
+        if name in self._listOfFunctionDefinitions:
+            self._listOfFunctionDefinitions.pop(name)
 
     def delete_all_function_definitions(self):
         """
@@ -840,7 +894,7 @@ class Model(SortableObject, Jsonify):
             return self.get_assignment_rule(ename)
         if ename in self.listOfFunctionDefinitions:
             return self.get_function_definition(ename)
-        return 'Element not found!'
+        raise ModelError(f"model.get_element(): element={ename} not found")
 
 
     def get_best_solver(self):
@@ -856,9 +910,12 @@ class Model(SortableObject, Jsonify):
         """
         from gillespy2.solvers.numpy import can_use_numpy
         hybrid_check = False
-        if len(self.get_all_assignment_rules()) or len(self.get_all_rate_rules())  \
-                or len(self.get_all_function_definitions()) or len(self.get_all_events()):
+        chybrid_check = True
+        if len(self.get_all_rate_rules())  or len(self.get_all_events()):
             hybrid_check = True
+        if len(self.get_all_assignment_rules()) or len(self.get_all_function_definitions()):
+            hybrid_check = True
+            chybrid_check = False
 
         if len(self.get_all_species()) and hybrid_check == False:
             for i in self.get_all_species():
@@ -866,18 +923,20 @@ class Model(SortableObject, Jsonify):
                 if tempMode == 'dynamic' or tempMode == 'continuous':
                     hybrid_check = True
                     break
-        if can_use_numpy and hybrid_check:
-            from gillespy2 import TauHybridSolver
-            return TauHybridSolver
 
-        elif not can_use_numpy and hybrid_check:
-            raise ModelError('TauHybridSolver is the only solver currently that supports '
-                             'AssignmentRules, RateRules, FunctionDefinitions, or Events. '
-                             'Please install Numpy.')
-        
         from gillespy2.solvers.cpp.build.build_engine import BuildEngine
         can_use_cpp = not len(BuildEngine.get_missing_dependencies())
 
+        if not can_use_cpp and not can_use_numpy:
+            raise ModelError('Dependency Error, cannot run model.')
+
+        if can_use_cpp and hybrid_check and chybrid_check:
+            from gillespy2 import TauHybridCSolver
+            return TauHybridCSolver
+        elif can_use_numpy and hybrid_check:
+            from gillespy2 import TauHybridSolver
+            return TauHybridSolver
+        
         if can_use_cpp is False and can_use_numpy and not hybrid_check:
             from gillespy2 import NumPySSASolver
             return NumPySSASolver
@@ -893,6 +952,9 @@ class Model(SortableObject, Jsonify):
         from gillespy2.solvers.numpy import can_use_numpy
         from gillespy2.solvers.cpp.build.build_engine import BuildEngine
         can_use_cpp = not len(BuildEngine.get_missing_dependencies())
+        chybrid_check = True
+        if len(self.get_all_assignment_rules()) or len(self.get_all_function_definitions()):
+            chybrid_check = False
 
         if not can_use_cpp and can_use_numpy:
             raise ModelError("Please install C++ or Numpy to use GillesPy2 solvers.")
@@ -920,11 +982,42 @@ class Model(SortableObject, Jsonify):
             else:
                 from gillespy2 import ODESolver
                 return ODESolver
+
+        elif algorithm == 'Tau-Hybrid':
+            if can_use_cpp and chybrid_check:
+                from gillespy2 import TauHybridCSolver
+                return TauHybridCSolver
+            else:
+                from gillespy2 import TauHybridSolver
+                return TauHybridSolver
+
+        elif algorithm == 'CLE':
+            from gillespy2 import CLESolver
+            return CLESolver
+            
         else:
             raise ModelError("Invalid value for the argument 'algorithm' entered. "
-                             "Please enter 'SSA', 'ODE', or 'Tau-leaping'.")
+                             "Please enter 'SSA', 'ODE', 'CLE', 'Tau-leaping', or 'Tau-Hybrid'.")
 
-    def run(self, solver=None, timeout=0, t=None, increment=None, show_labels=True, cpp_support=False, algorithm=None,
+    def get_model_features(self) -> "Set[Type]":
+        """
+        Determine what solver-specific model features are present on the model.
+        Used to validate that the model is compatible with the given solver.
+
+        :returns: Set containing the classes of every solver-specific feature present on the model.
+        """
+        features = set()
+        if len(self.listOfEvents):
+            features.add(gillespy2.Event)
+        if len(self.listOfRateRules):
+            features.add(gillespy2.RateRule)
+        if len(self.listOfAssignmentRules):
+            features.add(gillespy2.AssignmentRule)
+        if len(self.listOfFunctionDefinitions):
+            features.add(gillespy2.FunctionDefinition)
+        return features
+
+    def run(self, solver=None, timeout=0, t=None, increment=None, show_labels=True, algorithm=None,
             **solver_args):
         """
         Function calling simulation of the model. There are a number of
@@ -942,10 +1035,6 @@ class Model(SortableObject, Jsonify):
 
         :param solver_args: Solver-specific arguments to be passed to solver.run()
 
-        :param cpp_support: INTERNAL USE ONLY, flag for whether or not a computer has the capability to compile a
-            C++ program.
-        :type cpp_support: bool
-
         :param algorithm: Specify algorithm ('ODE', 'Tau-Leaping', or 'SSA') for GillesPy2 to automatically pick best solver using that algorithm.
         :type algorithm: str
 
@@ -962,11 +1051,7 @@ class Model(SortableObject, Jsonify):
 
         if not show_labels:
             from gillespy2.core import log
-            log.warning('show_labels = False is deprecated. Future releases '
-                        'of GillesPy2 may not support this feature.')
-
-        if t is None:
-            t = self.tspan[-1]
+            log.warning('show_labels = False is deprecated. Future releases of GillesPy2 may not support this feature.')
 
         if solver is None:
             if algorithm is not None:
@@ -974,30 +1059,22 @@ class Model(SortableObject, Jsonify):
             else:
                 solver = self.get_best_solver()
 
-        if self.user_set_tspan and increment is not None:
-            raise SimulationError(
-                """
-                Failed while preparing to run the model. Both increment and timespan are set.
-
-                To continue either remove your `timespan` definition from your Model or remove the 
-                `increment` argument from this `model.run()` call.               
-                """
-            )
+        if not hasattr(solver, "is_instantiated"):
+            try:
+                sol_kwargs = {'model': self}
+                if "CSolver" in solver.name and \
+                    ("resume" in solver_args or "variables" in solver_args or "live_output" in solver_args):
+                    sol_kwargs['variable'] = True
+                solver = solver(**sol_kwargs)
+            except Exception as err:
+                raise SimulationError(f"{solver} is not a valid solver.  Reason Given: {err}.") from err
 
         try:
-            return solver.run(model=self, t=t, increment=increment, timeout=timeout, **solver_args)
+            return solver.run(t=t, increment=increment, timeout=timeout, **solver_args)
         except Exception as e:
-            # If user has specified the SSACSolver, but they don't actually have a g++ compiler,
-            # This will throw an error and throw log. IF a user specifies cpp_support == True and don't have a compiler
-            # They would bypass this log.warning and just recieve an error
-            if cpp_support is False and not isinstance(solver, str):
-                if solver.name == 'SSACSolver':
-                    from gillespy2.core import log
-                    log.warning("Please install/configure 'g++' and 'make' on your"
-                                " system, to ensure that GillesPy2 C solvers will"
-                                " run properly.")
             raise SimulationError(
-                "argument 'solver={}' to run() failed.  Reason Given: {}".format(solver, e))
+                "argument 'solver={}' to run() failed.  Reason Given: {}".format(solver, e)
+            ) from e
 
 
 class StochMLDocument():
