@@ -1,6 +1,6 @@
 /*
  * GillesPy2 is a modeling toolkit for biochemical simulation.
- * Copyright (C) 2019-2021 GillesPy2 developers.
+ * Copyright (C) 2019-2022 GillesPy2 developers.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,26 @@
 #include <vector>
 #include <iostream>
 
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
+#include <windows.h>
+#define GPY_PID_GET() ((int) GetCurrentProcessId())
+#define GPY_INTERRUPT_HANDLER(handler_name, handler_code) \
+static BOOL WINAPI handler_name(DWORD signum) {           \
+	do handler_code while(0);                             \
+	return TRUE;										  \
+}
+#define GPY_INTERRUPT_INSTALL_HANDLER(handler) SetConsoleCtrlHandler(handler, TRUE)
+#else
+#include <unistd.h>
+#include <csignal>
+#define GPY_PID_GET() (getpid())
+#define GPY_INTERRUPT_HANDLER(handler_name, handler_code) \
+static void handler_name(int signum) {                    \
+	do handler_code while(0);                             \
+}
+#define GPY_INTERRUPT_INSTALL_HANDLER(handler) signal(SIGINT, handler)
+#endif
+
 namespace Gillespy
 {
 	typedef unsigned int ReactionId;
@@ -137,8 +157,6 @@ namespace Gillespy
 		double end_time;
 		double *timeline;
 
-		PType *trajectories_1D;
-		PType ***trajectories;
 		PType *current_state;
 
 		Model<PType> *model;
@@ -183,11 +201,73 @@ namespace Gillespy
 		/// \param os Output stream to write the final buffer contents to.
 		void output_buffer_final(std::ostream &os);
 
+		/// \name set_status
+		///
+		/// \brief Sets the return code of the simulation program.
+		/// \param status Opaque return code value.
+		inline void set_status(uint8_t status) { m_status = status; }
+		inline uint8_t get_status() { return m_status; }
+
 		~Simulation();
 
+		enum SimulationStatus : uint8_t
+		{
+			OK = 0,
+			PAUSED = 33,
+		};
+
 	private:
 		unsigned int last_timestep;
 		unsigned int trajectory_num;
+		uint8_t m_status = 0;
+	};
+
+	enum class LogLevel
+	{
+		INFO   = 0,
+		WARN   = 1,
+		ERR    = 2,
+		CRIT   = 3,
+		SILENT = 4,
+	};
+
+	class LogStream
+	{
+	public:
+		virtual LogStream &operator<<(std::ostream &(*output)(std::ostream&)) { return *this; }
+		virtual LogStream &operator<<(const char *output) { return *this; }
+		virtual LogStream &operator<<(int output) { return *this; }
+		virtual LogStream &operator<<(double output) { return *this; }
+	};
+
+	class StdErrLogStream : public LogStream
+	{
+	public:
+		LogStream &operator<<(std::ostream &(*output)(std::ostream&)) override { std::cerr << output; return *this; }
+		LogStream &operator<<(const char *output) override { std::cerr << output; return *this; }
+		LogStream &operator<<(int output) override { std::cerr << output; return *this; }
+		LogStream &operator<<(double output) override { std::cerr << output; return *this; }
+	};
+
+	class Logger
+	{
+		LogLevel m_log_level = LogLevel::CRIT;
+		LogStream m_null = LogStream();
+		StdErrLogStream m_stderr = StdErrLogStream();
+
+	public:
+		LogStream &info();
+		LogStream &warn();
+		LogStream &err();
+		LogStream &crit();
+
+		inline LogLevel get_log_level() { return m_log_level; }
+		inline LogLevel set_log_level(LogLevel log_level)
+		{
+			LogLevel previous = m_log_level;
+			m_log_level = log_level;
+			return previous;
+		}
 	};
 
 	template <typename TNum>
