@@ -1,6 +1,6 @@
 /*
  * GillesPy2 is a modeling toolkit for biochemical simulation.
- * Copyright (C) 2019-2021 GillesPy2 developers.
+ * Copyright (C) 2019-2022 GillesPy2 developers.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ int main(int argc, char* argv[])
 	add_reactions(model);
 
 	if(seed_time){
-		random_seed = time(NULL);
+		random_seed = time(nullptr) % GPY_PID_GET();
 	}
 	//Simulation INIT
 	TauHybrid::HybridSimulation simulation(model);
@@ -69,8 +69,11 @@ int main(int argc, char* argv[])
 
 	std::vector<Gillespy::TauHybrid::Event> events;
 	Gillespy::TauHybrid::Event::use_events(events);
+	Logger logger;
+	if (parser.verbose)
+		logger.set_log_level(LogLevel::INFO);
 
-	TauHybrid::TauHybridCSolver(&simulation, events, tau_tol);
-	simulation.output_results_buffer(std::cout);
-	return 0;
+	TauHybrid::TauHybridCSolver(&simulation, events, tau_tol, logger);
+	simulation.output_buffer_final(std::cout);
+	return simulation.get_status();
 }
