@@ -1,6 +1,6 @@
 /*
  * GillesPy2 is a modeling toolkit for biochemical simulation.
- * Copyright (C) 2019-2021 GillesPy2 developers.
+ * Copyright (C) 2019-2022 GillesPy2 developers.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,24 +32,30 @@
 #include "integrator.h"
 #include "tau.h"
 
+static void silent_error_handler(int error_code, const char *module, const char *function_name,
+						  char *message, void *eh_data);
+
 namespace Gillespy
 {
+	static volatile bool interrupted = false;
+
+	GPY_INTERRUPT_HANDLER(signal_handler, {
+		interrupted = true;
+	})
+
 	namespace TauHybrid
 	{
-
-		bool interrupted = false;
-
-		void signalHandler(int signum)
-		{
-			interrupted = true;
-		}
-
-		void TauHybridCSolver(HybridSimulation *simulation, std::vector<Event> &events, const double tau_tol)
+		void TauHybridCSolver(
+				HybridSimulation *simulation,
+				std::vector<Event> &events,
+				const double tau_tol,
+				Logger &logger)
 		{
 			if (simulation == NULL)
 			{
 				return;
 			}
+			GPY_INTERRUPT_INSTALL_HANDLER(signal_handler);
 
 			Model<double> &model = *(simulation->model);
 			int num_species = model.number_species;
@@ -72,12 +78,16 @@ namespace Gillespy
 				y = y0;
 			}
 			Integrator sol(simulation, y, GPY_HYBRID_RELTOL, GPY_HYBRID_ABSTOL);
+			if (logger.get_log_level() == LogLevel::CRIT)
+			{
+				sol.set_error_handler(silent_error_handler);
+			}
 
 			// Tau selector initialization. Used to select a valid tau step.
 			TauArgs<double> tau_args = initialize(model, tau_tol);
 
 			// Simulate for each trajectory
-			for (int traj = 0; traj < num_trajectories; traj++)
+			for (int traj = 0; !interrupted && traj < num_trajectories; traj++)
 			{
 				if (traj > 0)
 				{
@@ -88,14 +98,15 @@ namespace Gillespy
 				// TODO: change back double -> hybrid_state, once we figure out how that works
 				EventList event_list;
 				std::vector<double> current_state(num_species);
-				std::vector<int> current_populations(num_species);
 
 				// Initialize the species population for the trajectory.
 				for (int spec_i = 0; spec_i < num_species; ++spec_i)
 				{
 					current_state[spec_i] = species[spec_i].initial_population;
-					current_populations[spec_i] = species[spec_i].initial_population;
+					simulation->current_state[spec_i] = current_state[spec_i];
 				}
+				simulation->reset_output_buffer(traj);
+				simulation->output_buffer_range(std::cout);
 
 				// Check for initial event triggers at t=0 (based on initial_value of trigger)
 				std::set<int> event_roots;
@@ -115,7 +126,6 @@ namespace Gillespy
 					spec->partition_mode = spec->user_mode == SimulationState::DYNAMIC
 										   ? SimulationState::DISCRETE
 										   : spec->user_mode;
-					simulation->trajectories[traj][0][spec_i] = current_state[spec_i];
 				}
 
 				// SIMULATION STEP LOOP
@@ -132,40 +142,28 @@ namespace Gillespy
 				// An invalid simulation state indicates that an unrecoverable error has occurred,
 				//   and the trajectory should terminate early.
 				bool invalid_state = false;
-				// This is a temporary fix. Ideally, invalid state should allow for integrator options change.
-				// For now, a "guard" is put in place to prevent potentially infinite loops from occurring.
-				unsigned int integration_guard = 1000;
 
-				while (integration_guard > 0 && simulation->current_time < simulation->end_time)
+
+				while (!interrupted && !invalid_state && simulation->current_time < simulation->end_time)
 				{
 					// Compute current propensity values based on existing state.
 					for (unsigned int rxn_i = 0; rxn_i < num_reactions; ++rxn_i)
 					{
 						HybridReaction &rxn = simulation->reaction_state[rxn_i];
-						double propensity = 0.0;
-						switch (rxn.mode)
-						{
-						case SimulationState::CONTINUOUS:
-							propensity = Reaction::propensity(rxn_i, current_state.data());
-							break;
-						case SimulationState::DISCRETE:
-							propensity = Reaction::propensity(rxn_i, current_populations.data());
-							break;
-						default:
-							break;
-						}
-						sol.data.propensities[rxn_i] = propensity;
+						sol.data.propensities[rxn_i] = rxn.ssa_propensity(current_state.data());
 					}
+					if (interrupted)
+						break;
 
 					// Expected tau step is determined.
-					tau_step = select(
+					tau_step = select<double, double>(
 							model,
 							tau_args,
 							tau_tol,
 							simulation->current_time,
 							save_time,
 							sol.data.propensities,
-							current_populations
+							current_state
 					);
 					partition_species(
 							simulation->reaction_state,
@@ -187,10 +185,17 @@ namespace Gillespy
 					// If a retry with a smaller tau_step is deemed necessary, this will change.
 					next_time = simulation->current_time + tau_step;
 
+					// Ensure that any previous changes to the current state is reflected by the integrator.
+					std::copy(current_state.begin(), current_state.end(), N_VGetArrayPointer(sol.y));
+
 					// The integration loop continues until a valid solution is found.
 					// Any invalid Tau steps (which cause negative populations) are discarded.
 					sol.save_state();
 
+					// This is a temporary fix. Ideally, invalid state should allow for integrator options change.
+					// For now, a "guard" is put in place to prevent potentially infinite loops from occurring.
+					unsigned int integration_guard = 1000;
+
 					do
 					{
 						invalid_state = false;
@@ -202,79 +207,84 @@ namespace Gillespy
 						if (sol.status == IntegrationStatus::BAD_STEP_SIZE)
 						{
 							invalid_state = true;
-							// Breaking early causes `invalid_state` to remain set,
-							//   resulting in an early termination of the trajectory.
-							break;
-						}
-
-						// The integrator has, at this point, been validated.
-						// Any errors beyond this point is assumed to be a stochastic state failure.
-
-						// 0-initialize our population_changes array.
-						for (int p_i = 0; p_i < num_species; ++p_i)
-						{
-							population_changes[p_i] = 0;
-						}
-
-						// Start with the species concentration as a baseline value.
-						// Stochastic reactions will update populations relative to their concentrations.
-						for (int spec_i = 0; spec_i < num_species; ++spec_i)
-						{
-							current_state[spec_i] = result.concentrations[spec_i];
-						}
-
-						if (!rxn_roots.empty())
-						{
-							// "Direct" roots found; these are executed manually
-							for (unsigned int rxn_i : rxn_roots)
-							{
-								// "Fire" a reaction by recording changes in dependent species.
-								// If a negative value is detected, break without saving changes.
-								for (int spec_i = 0; spec_i < num_species; ++spec_i)
-								{
-									// Unlike the Tau-leaping version of reaction firings,
-									// it is not possible to have a negative state occur in direct reactions.
-									population_changes[spec_i] += model.reactions[rxn_i].species_change[spec_i];
-									result.reactions[rxn_i] = log(urn.next());
-								}
-							}
-							rxn_roots.clear();
 						}
 						else
 						{
-							// The newly-updated reaction_states vector may need to be reconciled now.
-							// A positive reaction_state means reactions have potentially fired.
-							// NOTE: it is possible for a population to swing negative, where a smaller Tau is needed.
-							for (int rxn_i = 0; rxn_i < num_reactions; ++rxn_i)
+							// The integrator has, at this point, been validated.
+							// Any errors beyond this point is assumed to be a stochastic state failure.
+
+							// 0-initialize our population_changes array.
+							for (int p_i = 0; p_i < num_species; ++p_i)
 							{
-								// Temporary variable for the reaction's state.
-								// Does not get updated unless the changes are deemed valid.
-								double rxn_state = result.reactions[rxn_i];
+								population_changes[p_i] = 0;
+							}
 
-								switch (simulation->reaction_state[rxn_i].mode)
+							// Start with the species concentration as a baseline value.
+							// Stochastic reactions will update populations relative to their concentrations.
+							for (int spec_i = 0; spec_i < num_species; ++spec_i)
+							{
+								current_state[spec_i] = result.concentrations[spec_i];
+							}
+
+							if (!rxn_roots.empty())
+							{
+								// "Direct" roots found; these are executed manually
+								for (unsigned int rxn_i : rxn_roots)
 								{
-								case SimulationState::DISCRETE:
-									while (rxn_state >= 0)
+									// "Fire" a reaction by recording changes in dependent species.
+									// If a negative value is detected, break without saving changes.
+									for (int spec_i = 0; spec_i < num_species; ++spec_i)
 									{
-										// "Fire" a reaction by recording changes in dependent species.
-										// If a negative value is detected, break without saving changes.
-										for (int spec_i = 0; spec_i < num_species; ++spec_i)
-										{
-											population_changes[spec_i] +=
-													model.reactions[rxn_i].species_change[spec_i];
-											if (current_state[spec_i] + population_changes[spec_i] < 0)
-											{
-												invalid_state = true;
-											}
-										}
-
-										rxn_state += log(urn.next());
+										// Unlike the Tau-leaping version of reaction firings,
+										// it is not possible to have a negative state occur in direct reactions.
+										population_changes[spec_i] += model.reactions[rxn_i].species_change[spec_i];
+										result.reactions[rxn_i] = log(urn.next());
 									}
-									result.reactions[rxn_i] = rxn_state;
-									break;
+								}
+								rxn_roots.clear();
+							}
+							else
+							{
+								// The newly-updated reaction_states vector may need to be reconciled now.
+								// A positive reaction_state means reactions have potentially fired.
+								// NOTE: it is possible for a population to swing negative, where a smaller Tau is needed.
+								for (int rxn_i = 0; rxn_i < num_reactions; ++rxn_i)
+								{
+									// Temporary variable for the reaction's state.
+									// Does not get updated unless the changes are deemed valid.
+									double rxn_state = result.reactions[rxn_i];
 
-								case SimulationState::CONTINUOUS:
-								default:
+									switch (simulation->reaction_state[rxn_i].mode)
+									{
+									case SimulationState::DISCRETE:
+										while (rxn_state >= 0)
+										{
+											// "Fire" a reaction by recording changes in dependent species.
+											// If a negative value is detected, break without saving changes.
+											for (int spec_i = 0; spec_i < num_species; ++spec_i)
+											{
+												population_changes[spec_i] +=
+														model.reactions[rxn_i].species_change[spec_i];
+											}
+
+											rxn_state += log(urn.next());
+										}
+										result.reactions[rxn_i] = rxn_state;
+										break;
+
+									case SimulationState::CONTINUOUS:
+									default:
+										break;
+									}
+								}
+							}
+
+							// Explicitly check for invalid population state, now that changes have been tallied.
+							for (int spec_i = 0; spec_i < num_species; ++spec_i)
+							{
+								if (current_state[spec_i] + population_changes[spec_i] < 0)
+								{
+									invalid_state = true;
 									break;
 								}
 							}
@@ -284,6 +294,13 @@ namespace Gillespy
 						// Only update state with the given population changes if valid.
 						if (invalid_state)
 						{
+							if (--integration_guard == 0)
+							{
+								// Breaking early causes `invalid_state` to remain set,
+								//   resulting in an early termination of the trajectory.
+								break;
+							}
+
 							sol.restore_state();
 							tau_step /= 2;
 							next_time = simulation->current_time + tau_step;
@@ -301,17 +318,24 @@ namespace Gillespy
 									current_state[p_i] += population_changes[p_i];
 									result.concentrations[p_i] = current_state[p_i];
 								}
-								current_populations[p_i] = (int) current_state[p_i];
 							}
 						}
-					} while (invalid_state);
+					} while (invalid_state && !interrupted);
 
-					// Invalid state after the do-while loop implies that an unrecoverable error has occurred.
-					// While prior results are considered usable, the current integration results are not.
-					// Calling `continue` with an invalid state will discard the results and terminate the trajectory.
-					integration_guard = invalid_state
-										? integration_guard - 1
-										: 1000;
+					if (interrupted)
+						break;
+					else if (invalid_state)
+					{
+						// Invalid state after the do-while loop implies that an unrecoverable error has occurred.
+						// While prior results are considered usable, the current integration results are not.
+						// Calling `continue` with an invalid state will discard the results and terminate the trajectory.
+						logger.err()
+								<< "[Trajectory #" << traj << "] "
+								<< "Integration guard triggered; problem space too stiff at t="
+								<< simulation->current_time << std::endl;
+						simulation->set_status(HybridSimulation::LOOP_OVER_INTEGRATE);
+						continue;
+					}
 
 					// ===== <EVENT HANDLING> =====
 					if (!event_list.has_active_events())
@@ -344,23 +368,27 @@ namespace Gillespy
 					{
 						for (int spec_i = 0; spec_i < num_species; ++spec_i)
 						{
-							simulation->trajectories[traj][save_idx][spec_i] = current_state[spec_i];
+							simulation->current_state[spec_i] = current_state[spec_i];
 						}
-						save_time = simulation->timeline[++save_idx];
+						simulation->output_buffer_range(std::cout, save_idx++);
+						save_time = simulation->timeline[save_idx];
 					}
 				}
 
-				if (integration_guard == 0)
-				{
-					std::cerr
-							<< "[Trajectory #" << traj << "] "
-							<< "Integration guard triggered; problem space too stiff at t="
-							<< simulation->current_time << std::endl;
-				}
-
 				// End of trajectory
 				delete[] population_changes;
 			}
+
+			if (interrupted)
+			{
+				simulation->set_status(HybridSimulation::OK);
+			}
 		}
 	}
 }
+
+void silent_error_handler(int error_code, const char *module, const char *function_name,
+						  char *message, void *eh_data)
+{
+	// Do nothing
+}
