@@ -1,6 +1,6 @@
 /*
  * GillesPy2 is a modeling toolkit for biochemical simulation.
- * Copyright (C) 2019-2021 GillesPy2 developers.
+ * Copyright (C) 2019-2022 GillesPy2 developers.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,6 @@ namespace Gillespy
 {
 	namespace TauHybrid
 	{
-    	void TauHybridCSolver(HybridSimulation* simulation, std::vector<Event> &events, double tau_tol);
+    	void TauHybridCSolver(HybridSimulation* simulation, std::vector<Event> &events, double tau_tol, Logger &logger);
 	}
 }
