@@ -1,6 +1,6 @@
 /*
  * GillesPy2 is a modeling toolkit for biochemical simulation.
- * Copyright (C) 2019-2021 GillesPy2 developers.
+ * Copyright (C) 2019-2022 GillesPy2 developers.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,13 @@ namespace Gillespy
 	double map_propensity(unsigned int reaction_id, unsigned int *S, double *parameters, const double *constants);
 	double map_propensity(unsigned int reaction_id, double *S, double *parameters, const double *constants);
 
+	double map_ssa_propensity(unsigned int reaction_id, int *state, double *parameters, const double *constants);
+	double map_ssa_propensity(unsigned int reaction_id, unsigned int *state, double *parameters, const double *constants);
+	double map_ssa_propensity(unsigned int reaction_id, double *state, double *parameters, const double *constants);
+	double map_ode_propensity(unsigned int reaction_id, int *state, double *parameters, const double *constants);
+	double map_ode_propensity(unsigned int reaction_id, unsigned int *state, double *parameters, const double *constants);
+	double map_ode_propensity(unsigned int reaction_id, double *state, double *parameters, const double *constants);
+
 	template <typename T>
 	void add_reactions(Model<T> &model);
 
