@@ -1,8 +1,11 @@
+import numpy as np
 import gillespy2
 from gillespy2.solvers.cpp.c_decoder import IterativeSimDecoder
 from gillespy2.solvers.utilities import solverutils as cutils
-from gillespy2.core import GillesPySolver, gillespyError, Model
+from gillespy2.core import GillesPySolver, Model, Event, RateRule
+from gillespy2.core.gillespyError import *
 from typing import Union
+from enum import IntEnum
 from gillespy2.core import Results
 
 from .c_solver import CSolver, SimulationReturnCode
@@ -12,6 +15,9 @@ class TauHybridCSolver(GillesPySolver, CSolver):
     name = "TauHybridCSolver"
     target = "hybrid"
 
+    class ErrorStatus(IntEnum):
+        LOOP_OVER_INTEGRATE = 1
+
     @classmethod
     def __create_options(cls, sanitized_model: "SanitizedModel") -> "SanitizedModel":
         """
@@ -92,7 +98,8 @@ class TauHybridCSolver(GillesPySolver, CSolver):
                     elif variable in sanitized_model.model.listOfParameters:
                         variable = sanitized_model.model.listOfParameters.get(variable)
                     else:
-                        raise ValueError(f"Invalid event assignment {assign}: received name {variable} "
+                        raise ValueError(f"Error in event={event} "
+                                         f"Invalid event assignment {assign}: received name {variable} "
                                          f"Must match the name of a valid Species or Parameter.")
 
                 if isinstance(variable, gillespy2.Species):
@@ -107,6 +114,17 @@ class TauHybridCSolver(GillesPySolver, CSolver):
                 assignments.append(str(assign_id))
                 event_assignment_list.append(assign_str)
                 assign_id += 1
+            # Check for "None"s
+            for a in assignments:
+                if a is None: raise Exception(f"assignment={a} is None in event={event}")
+            if event_id is None: raise Exception(f"event_id is None in event={event}")
+            if trigger is None: raise Exception(f"trigger is None in event={event}")
+            if delay is None: raise Exception(f"delay is None in event={event}")
+            if priority is None: raise Exception(f"priority is None in event={event}")
+            if use_trigger is None: raise Exception(f"use_trigger is None in event={event}")
+            if use_persist is None: raise Exception(f"use_persist is None in event={event}")
+            if initial_value is None: raise Exception(f"initial_value is None in event={event}")
+
             assignments: "str" = " AND ".join(assignments)
             event_list.append(
                 f"EVENT("
@@ -128,6 +146,13 @@ class TauHybridCSolver(GillesPySolver, CSolver):
         sanitized_model.options["GPY_HYBRID_NUM_EVENT_ASSIGNMENTS"] = str(len(event_assignment_list))
         return sanitized_model
 
+    @classmethod
+    def get_supported_features(cls):
+        return {
+            Event,
+            RateRule,
+        }
+
     def _build(self, model: "Union[Model, SanitizedModel]", simulation_name: str, variable: bool, debug: bool = False,
                custom_definitions=None) -> str:
         variable = variable or len(model.listOfEvents) > 0
@@ -136,37 +161,60 @@ class TauHybridCSolver(GillesPySolver, CSolver):
             sanitized_model.use_rate_rule(rate_rule)
         return super()._build(sanitized_model, simulation_name, variable, debug)
 
+    def _handle_return_code(self, return_code: "int") -> "int":
+        if return_code == TauHybridCSolver.ErrorStatus.LOOP_OVER_INTEGRATE:
+            raise ExecutionError("Loop over integrate exceeded, problem space is too stiff")
+        return super()._handle_return_code(return_code)
+
     def get_solver_settings(self):
         """
         :return: Tuple of strings, denoting all keyword argument for this solvers run() method.
         """
         return ('model', 't', 'number_of_trajectories', 'timeout', 'increment', 'seed', 'debug', 'profile')
 
-    def run(self=None, model: Model = None, t: int = 20, number_of_trajectories: int = 1, timeout: int = 0,
-            increment: int = None, seed: int = None, debug: bool = False, profile: bool = False, variables={}, 
+    def run(self=None, model: Model = None, t: int = None, number_of_trajectories: int = 1, timeout: int = 0,
+            increment: int = None, seed: int = None, debug: bool = False, profile: bool = False, variables={},
             resume=None, live_output: str = None, live_output_options: dict = {}, tau_step: int = .03, tau_tol=0.03, **kwargs):
 
+        from gillespy2 import log
+
         if self is None:
+            # Post deprecation block
+            # raise SimulationError("TauHybridCSolver must be instantiated to run the simulation")
+            # Pre deprecation block
+            log.warning(
+                """
+                `gillespy2.Model.run(solver=TauHybridCSolver)` is deprecated.
+
+                You should use `gillespy2.Model.run(solver=TauHybridCSolver(model=gillespy2.Model))
+                Future releases of GillesPy2 may not support this feature.
+                """
+            )
             self = TauHybridCSolver(model, resume=resume)
+
+        if model is not None:
+            log.warning('model = gillespy2.model is deprecated. Future releases '
+                        'of GillesPy2 may not support this feature.')
         if self.model is None:
             if model is None:
                 raise SimulationError("A model is required to run the simulation.")
             self._set_model(model=model)
-        if model is not None and model.get_json_hash() != self.model.get_json_hash():
-            raise SimulationError("Model must equal TauHybridCSolver.model.")
-        self.model.resolve_parameters()
 
-        increment = self.get_increment(increment=increment)
+        self.model.resolve_parameters()
+        self.validate_model(self.model, model)
+        self.validate_sbml_features(model=self.model)
+
+        self.validate_tspan(increment=increment, t=t)
+        if increment is None:
+            increment = self.model.tspan[-1] - self.model.tspan[-2]
+        if t is None:
+            t = self.model.tspan[-1]
 
         # Validate parameters prior to running the model.
         self._validate_type(variables, dict, "'variables' argument must be a dictionary.")
         self._validate_variables_in_set(variables, self.species + self.parameters)
         self._validate_resume(t, resume)
         self._validate_kwargs(**kwargs)
-        self._validate_sbml_features({
-            "Assignment Rules": len(self.model.listOfAssignmentRules),
-            "Function Definitions": len(self.model.listOfFunctionDefinitions)
-        })
 
         if resume is not None:
             t = abs(t - int(resume["time"][-1]))
@@ -206,15 +254,12 @@ class TauHybridCSolver(GillesPySolver, CSolver):
             display_args = None
 
         args = self._make_args(args)
+        if debug:
+            args.append("--verbose")
         decoder = IterativeSimDecoder.create_default(number_of_trajectories, number_timesteps, len(self.model.listOfSpecies))
 
         sim_exec = self._build(self.model, self.target, self.variable, False)
         sim_status = self._run(sim_exec, args, decoder, timeout, display_args)
-
-        if sim_status == SimulationReturnCode.FAILED:
-            raise gillespyError.ExecutionError("Error encountered while running simulation C++ file:\n"
-                f"Return code: {int(sim_status)}.\n")
-
         trajectories, time_stopped = decoder.get_output()
 
         simulation_data = self._format_output(trajectories)
