@@ -1,24 +1,23 @@
-"""
-GillesPy2 is a modeling toolkit for biochemical simulation.
-Copyright (C) 2019-2021 GillesPy2 developers.
+# GillesPy2 is a modeling toolkit for biochemical simulation.
+# Copyright (C) 2019-2022 GillesPy2 developers.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import ast  # for dependency graphing
 import numpy as np
 from gillespy2.core import log, Species
+from gillespy2.core import ModelError
 
 """
 NUMPY SOLVER UTILITIES BELOW
@@ -91,19 +90,19 @@ def update_species_init_values(listOfSpecies, species, variables, resume = None)
     populations = ''
     for i in range(len(species) - 1):
         if species[i] in variables:
-            populations += '{} '.format(int(variables[species[i]]))
+            populations += '{} '.format(float(variables[species[i]]))
         else:
             if resume is not None:
-                populations += '{} '.format(int(resume[species[i]][-1]))
+                populations += '{} '.format(float(resume[species[i]][-1]))
             else:
-                populations += '{} '.format(int(listOfSpecies[species[i]].initial_value))
+                populations += '{} '.format(float(listOfSpecies[species[i]].initial_value))
     if species[-1] in variables:
-        populations += '{}'.format(int(variables[species[-1]]))
+        populations += '{}'.format(float(variables[species[-1]]))
     else:
         if resume is not None:
-            populations += '{} '.format(int(resume[species[-1]][-1]))
+            populations += '{} '.format(float(resume[species[-1]][-1]))
         else:
-            populations += '{}'.format(int(listOfSpecies[species[-1]].initial_value))
+            populations += '{}'.format(float(listOfSpecies[species[-1]].initial_value))
     return populations
 
 def change_param_values(listOfParameters, parameters, volume, variables):
@@ -143,8 +142,11 @@ def species_parse(model, custom_prop_fun):
 
     class SpeciesParser(ast.NodeTransformer):
         def visit_Name(self, node):
-            if isinstance(model.get_element(node.id), Species):
-                parsed_species.append(model.get_element(node.id))
+            try:
+                if isinstance(model.get_element(node.id), Species):
+                    parsed_species.append(model.get_element(node.id))
+            except ModelError:
+                pass
 
     expr = custom_prop_fun
     expr = ast.parse(expr, mode='eval')
