@@ -1,20 +1,18 @@
-"""
-GillesPy2 is a modeling toolkit for biochemical simulation.
-Copyright (C) 2019-2021 GillesPy2 developers.
+# GillesPy2 is a modeling toolkit for biochemical simulation.
+# Copyright (C) 2019-2022 GillesPy2 developers.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 from example_models import RobustModel, Example, ExampleNoTspan
@@ -69,8 +67,9 @@ class TestModel(unittest.TestCase):
     def test_uniform_timespan(self):
         model = Model()
         model.timespan(np.linspace(0, 1, 100))
-        with self.assertRaises(InvalidModelError):
+        with self.assertRaises(TimespanError):
             model.timespan(np.array([0, 0.1, 0.5]))
+            model.run()
 
     def test_duplicate_parameter_names(self):
         model = Model()
@@ -137,20 +136,6 @@ class TestModel(unittest.TestCase):
             """
         )
 
-    def test_valid_initial_value_float(self):
-        species = Species('A', initial_value=1.5, mode='continuous')
-
-    def test_invalid_initial_value_float(self):
-        with self.assertRaises(ValueError):
-            species = Species('A', initial_value=1.5)
-
-    def test_valid_initial_value_negative(self):
-        species = Species('A', initial_value=-1, allow_negative_populations=True)
-
-    def test_invalid_initial_value_negative(self):
-        with self.assertRaises(ValueError):
-            species = Species('A', initial_value=-1)
-
     def test_reaction_invalid_reactant(self):
         model = Model()
         rate = Parameter(name='rate', expression=0.5)
@@ -222,7 +207,8 @@ class TestModel(unittest.TestCase):
         number_points = 11
         model.timespan(np.linspace(0, 1, number_points))
         from gillespy2.solvers.numpy.ssa_solver import NumPySSASolver
-        results = model.run(number_of_trajectories=1, solver=NumPySSASolver, seed=1)
+        solver = NumPySSASolver(model=model)
+        results = model.run(number_of_trajectories=1, solver=solver, seed=1)
         self.assertTrue(len(results['time']) == number_points)
         self.assertTrue(len(results[species1.name]) == number_points)
         self.assertTrue(len(results[species2.name]) == number_points)
@@ -304,16 +290,12 @@ class TestModel(unittest.TestCase):
             results = model.run(number_of_trajectories = 1, solver = 'non_solver', seed = 1)
 
     def test_model_init_population_false_and_volume_warninag(self):
-        with self.assertRaises(Warning):
+        with self.assertRaises(ModelError):
             model = Model(population = False, volume = 0.9)
 
     def test_model_init_custom_tspan(self):
         model = Model(tspan = np.linspace(0, 20, 401))
 
-    def test_parameter_init_unspecified_expression(self):
-        with self.assertRaises(ParameterError):
-            parameter = Parameter(name = 'parameter')
-
     def test_ode_propensity(self):
         model = Model()
         rate = Parameter(name='rate', expression=0.5)
@@ -332,16 +314,6 @@ class TestModel(unittest.TestCase):
         self.assertEqual(model.listOfReactions['r4'].ode_propensity_function, 't')
 
 
-    def test_species_setter(self):
-        sp1 = Species('A',initial_value=10)
-        sp1.set_initial_value(5)
-        self.assertEqual(sp1.initial_value,5)
-        with self.assertRaises(SpeciesError):
-            sp1.set_initial_value(-1)
-        sp2 = Species('B',initial_value=5,mode='discrete')
-        with self.assertRaises(SpeciesError):
-            sp2.set_initial_value(.5)
-
     def test_robust_model(self):
         try:
             model = RobustModel()
