@@ -1,20 +1,18 @@
-"""
-GillesPy2 is a modeling toolkit for biochemical simulation.
-Copyright (C) 2019-2021 GillesPy2 developers.
+# GillesPy2 is a modeling toolkit for biochemical simulation.
+# Copyright (C) 2019-2022 GillesPy2 developers.
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import unittest
 import tempfile
@@ -41,20 +39,24 @@ class TestSSACSolver(unittest.TestCase):
 
     def test_run_example(self):
         model = Example()
-        results = model.run(solver=SSACSolver)
+        solver = SSACSolver(model=model)
+        results = model.run(solver=solver)
 
     def test_run_example__with_increment_only(self):
         model = ExampleNoTspan()
-        results = SSACSolver.run(model=model, increment=0.2)
+        solver = SSACSolver(model=model)
+        results = solver.run(increment=0.2)
 
     def test_run_example__with_tspan_only(self):
         model = Example()
-        results = SSACSolver.run(model=model)
+        solver = SSACSolver(model=model)
+        results = solver.run()
 
     def test_run_example__with_tspan_and_increment(self):
         with self.assertRaises(SimulationError):
             model = Example()
-            results = SSACSolver.run(model=model, increment=0.2)
+            solver = SSACSolver(model=model)
+            results = solver.run(increment=0.2)
 
 
 if __name__ == '__main__':
